@@ -12,6 +12,10 @@ from MoinMoin import config, caching, user, util, wikiutil
 from MoinMoin.logfile import eventlog
 from MoinMoin.util import filesys, MoinMoinNoFooter, timefuncs
 
+def is_cache_exception(e):
+    args = e.args
+    return not (len(args) != 1 or args[0] != 'CacheNeedsUpdate')
+
 class Page:
     """Page - Manage an (immutable) page associated with a WikiName.
        To change a page's content, use the PageEditor class.
@@ -139,9 +143,12 @@ class Page:
 
         revstr = '%08d' % rev
         pagefile = os.path.join(pagedir, 'revisions', revstr)
-        exists = os.path.exists(pagefile)
-        if exists:
-            self._setRealPageName(pagedir)
+        if rev != 99999999:
+            exists = os.path.exists(pagefile)
+            if exists:
+                self._setRealPageName(pagedir)
+        else:
+            exists = False
         return pagefile, rev, exists
 
     def _setRealPageName(self, pagedir):
@@ -785,7 +792,7 @@ class Page:
                     # safe when passed to a function expecting a string)
                     return ""
                 else:
-                    raise er
+                    raise
 
             # read file content and make sure it is closed properly
             try:
@@ -942,15 +949,26 @@ class Page:
 
         return subscriber_list
 
-
     def send_raw(self):
         """ Output the raw page data (action=raw) """
-        self.request.http_headers(["Content-type: text/plain;charset=%s" % config.charset])
-        text = self.get_raw_body()
-        text = self.encodeTextMimeType(text)
-        self.request.write(text)
-        raise MoinMoinNoFooter
+        request = self.request
+        request.setHttpHeader("Content-type: text/plain; charset=%s" % config.charset)
+        if self.exists():
+            # use the correct last-modified value from the on-disk file
+            # to ensure cacheability where supported. Because we are sending
+            # RAW (file) content, the file mtime is correct as Last-Modified header.
+            request.setHttpHeader("Status: 200 OK")
+            request.setHttpHeader("Last-Modified: %s" % timefuncs.formathttpdate(os.path.getmtime(self._text_filename())))
+            text = self.get_raw_body()
+            text = self.encodeTextMimeType(text)
+        else:
+            request.setHttpHeader('Status: 404 NOTFOUND')
+            request.setResponseCode(404)
+            text = u"Page %s not found." % self.page_name
 
+        request.http_headers()
+        request.write(text)
+        raise MoinMoinNoFooter
 
     def send_page(self, request, msg=None, **keywords):
         """ Output the formatted page.
@@ -997,9 +1015,14 @@ class Page:
         if self.default_formatter:
             from MoinMoin.formatter.text_html import Formatter
             self.formatter = Formatter(request, store_pagelinks=1)
+
+        # save formatter
+        no_formatter = object()
+        old_formatter = getattr(request, "formatter", no_formatter)
+        request.formatter = self.formatter
+
         self.formatter.setPage(self)
         if self.hilite_re: self.formatter.set_highlight_re(self.hilite_re)
-        request.formatter = self.formatter
 
         # default is wiki markup
         pi_format = self.cfg.default_markup or "wiki"
@@ -1119,30 +1142,27 @@ class Page:
         doc_leader = self.formatter.startDocument(self.page_name)
         page_exists = self.exists()
         if not content_only:
-            # send the document leader
-
-            # need to inform caches that content changes
-            # based on cookie (even if we aren't sending one now)
-            request.setHttpHeader("Vary: Cookie")
-            # we include User-Agent because a bot might be denied and get no content
-            request.setHttpHeader("Vary: User-Agent")
-
-            # use "nocache" headers if we're using a method that
-            # is not simply "display", or if a user is logged in
-            # (which triggers personalisation features)
-
+            request.setHttpHeader("Content-Type: %s; charset=%s" % ('text/html', config.charset))
             if page_exists:
-                if not request.cacheable or request.user.valid:
-                    request.http_headers(request.nocache)
+                request.setHttpHeader('Status: 200 OK')
+                if not request.cacheable:
+                    # use "nocache" headers if we're using a method that is not simply "display"
+                    request.disableHttpCaching(level=2)
+                elif request.user.valid:
+                    # use nocache headers if a user is logged in (which triggers personalisation features)
+                    request.disableHttpCaching(level=1)
                 else:
-                    # use the correct last-modified value from the on-disk file
-                    # to ensure cacheability where supported
-                    request.http_headers(["Last-Modified: " +
-                         timefuncs.formathttpdate(os.path.getmtime(self._text_filename()))])
-
+                    # TODO: we need to know if a page generates dynamic content
+                    # if it does, we must not use the page file mtime as last modified value
+                    # XXX The following code is commented because it is incorrect for dynamic pages:
+                    #lastmod = os.path.getmtime(self._text_filename())
+                    #request.setHttpHeader("Last-Modified: %s" % timefuncs.formathttpdate(lastmod))
+                    pass
             else:
-                request.http_headers(['Status: 404 NOTFOUND'])
+                request.setHttpHeader('Status: 404 NOTFOUND')
                 request.setResponseCode(404)
+
+            request.http_headers()
             request.write(doc_leader)
 
             # send the page header
@@ -1261,6 +1281,14 @@ class Page:
 
         request.clock.stop('send_page')
 
+        # restore old formatter (hopefully we dont throw any exception
+        # that is catched again)
+        if old_formatter is no_formatter:
+            del request.formatter
+        else:
+            request.formatter = old_formatter
+
+
     def getFormatterName(self):
         """ Return a formatter name as used in the caching system
 
@@ -1317,11 +1345,15 @@ class Page:
             try:
                 code = self.loadCache(request)
                 self.execute(request, parser, code)
-            except 'CacheNeedsUpdate':
+            except Exception, e:
+                if not is_cache_exception(e):
+                    raise
                 try:
                     code = self.makeCache(request, parser)
                     self.execute(request, parser, code)
-                except 'CacheNeedsUpdate':
+                except Exception, e:
+                    if not is_cache_exception(e):
+                        raise
                     request.log('page cache failed after creation')
                     self.format(parser)
         
@@ -1340,26 +1372,31 @@ class Page:
         import MoinMoin
         if hasattr(MoinMoin, '__loader__'):
             __file__ = os.path.join(MoinMoin.__loader__.archive, 'dummy')
-        exec code
+        try:
+            exec code
+        except 'CacheNeedsUpdate':
+            raise Exception('CacheNeedsUpdate')
 
     def loadCache(self, request):
         """ Return page content cache or raises 'CacheNeedsUpdate' """
         cache = caching.CacheEntry(request, self, self.getFormatterName())
         attachmentsPath = self.getPagePath('attachments', check_create=0)
         if cache.needsUpdate(self._text_filename(), attachmentsPath):
-            raise 'CacheNeedsUpdate'
+            raise Exception('CacheNeedsUpdate')
         
         import marshal
         try:
             return marshal.loads(cache.content())
+        except "CacheNeedsUpdate": # convert old exception into a new one
+            raise Exception('CacheNeedsUpdate')
         except (EOFError, ValueError, TypeError):
             # Bad marshal data, must update the cache.
             # See http://docs.python.org/lib/module-marshal.html
-            raise 'CacheNeedsUpdate'
+            raise Exception('CacheNeedsUpdate')
         except Exception, err:
             request.log('fail to load "%s" cache: %s' % 
                         (self.page_name, str(err)))
-            raise 'CacheNeedsUpdate'
+            raise Exception('CacheNeedsUpdate')
 
     def makeCache(self, request, parser):
         """ Format content into code, update cache and return code """
@@ -1508,26 +1545,36 @@ class Page:
         More efficient now by using special pagelinks formatter and
         redirecting possible output into null file.
         """
-        request.clock.start('parsePagelinks')
+        pagename = self.page_name
+        if request.parsePageLinks_running.get(pagename, False):
+            return [] # avoid recursion
+
+        # remember we are already running this function for this page:
+        request.parsePageLinks_running[pagename] = True
+
+        request.clock.start('parsePageLinks')
+
         class Null:
-            def write(self, str): pass
+            def write(self, data):
+                pass
+
         request.redirect(Null())
-        request.mode_getpagelinks = 1
+        request.mode_getpagelinks += 1
         try:
             try:
                 from MoinMoin.formatter.pagelinks import Formatter
                 formatter = Formatter(request, store_pagelinks=1)
-                page = Page(request, self.page_name, formatter=formatter)
+                page = Page(request, pagename, formatter=formatter)
                 page.send_page(request, content_only=1)
             except:
                 import traceback
-                traceback.print_exc()
+                traceback.print_exc(200)
         finally:
-            request.mode_getpagelinks = 0
+            request.mode_getpagelinks -= 1
             request.redirect()
             if hasattr(request, '_fmt_hd_counters'):
                 del request._fmt_hd_counters
-            request.clock.stop('parsePagelinks')
+            request.clock.stop('parsePageLinks')
         return formatter.pagelinks
 
     def getCategories(self, request):
