@@ -542,8 +542,8 @@ class convert_tree(visitor):
             #    self.text.append("\n")
 
     def process_br(self, node):
-            self.text.append(self.new_line) # without this, std multi-line text below some heading misses a whitespace
-                                            # when it gets merged to float text, like word word wordword word word
+        self.text.append(self.new_line) # without this, std multi-line text below some heading misses a whitespace
+                                        # when it gets merged to float text, like word word wordword word word
 
     def process_heading(self, node):
         text = self.node_list_text_only(node.childNodes).strip()
@@ -708,6 +708,8 @@ class convert_tree(visitor):
                     self.process_list(i)
                 elif name == 'dl':
                     self.process_dl(i)
+                elif name == 'a':
+                    self.process_a(i)
                 elif name == 'img':
                     self.process_img(i)
                 elif name == 'div':
@@ -761,8 +763,8 @@ class convert_tree(visitor):
             command = ",,"
         elif name == 'sup':
             command = "^"
-        elif name == 'font':
-            command = "" # just throw away font settings
+        elif name in ('font', 'meta', ):
+            command = "" # just throw away unsupported elements
         else:
             raise ConvertError("process_inline: Don't support %s element" % name)
         
@@ -793,7 +795,7 @@ class convert_tree(visitor):
 
     def process_tt(self, node):
         text = self.node_list_text_only(node.childNodes).replace("\n", " ")
-        if node.getAttribute("css") == "backtick":
+        if node.getAttribute("class") == "backtick":
             self.text.append("`%s`" % text)
         else:
             self.text.append("{{{%s}}}" % text)
@@ -906,6 +908,8 @@ class convert_tree(visitor):
                 result.append('rowbgcolor="%s"' % value)
         if node.hasAttribute("style"):
             result.append('rowstyle="%s"' % node.getAttribute("style"))
+        if node.hasAttribute("class"):
+            result.append('rowclass="%s"' % node.getAttribute("class"))
         return " ".join(result).strip()
 
     def _cell_style(self, node):
@@ -978,6 +982,8 @@ class convert_tree(visitor):
                 elif name == 'caption':
                     self.process_caption(node, i, style)
                     style = ''
+                elif name  in ('col', 'colgroup', ):
+                    pass # we don't support these, but we just ignore them
                 else:
                     raise ConvertError("process_table: Don't support %s element" % name)
             #else:
@@ -1082,7 +1088,7 @@ class convert_tree(visitor):
             text = self.node_list_text_only(node.childNodes)
             text = text.replace("\n", " ").lstrip()
 
-             # interwiki link
+            # interwiki link
             if class_ == "interwiki":
                 wikitag, wikiurl, wikitail, err = wikiutil.resolve_wiki(
                     self.request, title + ":")
