@@ -12,6 +12,11 @@ from MoinMoin.util import pysupport
 
 modules = pysupport.getPackageModules(__file__)
 
+# Check whether we can emit a RSS feed (code stolen from wikitest.py).
+# Currently RSS is broken on plain Python, and works only when installing PyXML.
+import xml
+rss_supported = '_xmlplus' in xml.__file__
+
 
 class ThemeBase:
     """ Base class for themes 
@@ -134,38 +139,63 @@ class ThemeBase:
         admin inserted in the config file. Everything it enclosed inside
         a div with id="logo".
         
-        @param d: parameter dictionary
         @rtype: unicode
         @return: logo html
         """
+        html = u''
         if self.cfg.logo_string:
             pagename = wikiutil.getFrontPage(self.request).page_name
             pagename = wikiutil.quoteWikinameURL(pagename)
             logo = wikiutil.link_tag(self.request, pagename, self.cfg.logo_string)
             html = u'''<div id="logo">%s</div>''' % logo
-            return html
-        return u''
+        return html
+
+    def interwiki(self, d):
+        """ Assemble the interwiki name display, linking to page_front_page
+        
+        @param d: parameter dictionary
+        @rtype: string
+        @return: interwiki html
+        """
+        html = u''
+        if self.request.cfg.show_interwiki:
+            # Show our interwikiname or Self (and link to page_front_page)
+            pagename = wikiutil.getFrontPage(self.request).page_name
+            pagename = wikiutil.quoteWikinameURL(pagename)
+            link = wikiutil.link_tag(self.request, pagename, self.request.cfg.interwikiname or 'Self')
+            html = u'<div id="interwiki"><span>%s</span></div>' % link
+        return html
         
     def title(self, d):
-        """ Assemble the title
+        """ Assemble the title (now using breadcrumbs)
         
         @param d: parameter dictionary
         @rtype: string
         @return: title html
         """
         _ = self.request.getText
-        if d['title_link']:
-            content = ('<a title="%(title)s" href="%(href)s">%(text)s</a>') % {
+        content = []
+        if d['title_link']: # having a link means we have a (linked) pagename ONLY as title, not a message title
+                            # XXX this method is rather ugly and should be improved
+            curpage = ''
+            segments = d['page_name'].split('/') # was: title_text
+            for s in segments[:-1]:
+                curpage += s
+                content.append("<li>%s</li>" % Page(self.request, curpage).link_to(self.request, s))
+                curpage += '/'
+            content.append(('<li><a class="backlink" title="%(title)s" href="%(href)s">%(text)s</a></li>') % {
                 'title': _('Click to do a full-text search for this title'),
                 'href': d['title_link'],
-                'text': wikiutil.escape(d['title_text']),
-                }
+                'text': wikiutil.escape(segments[-1]),
+                })
         else:
-            content = wikiutil.escape(d['title_text'])
-        html = '''
-<h1 id="title">%s</h1>
-''' % content
+            content.append('<li>%s</li>' % wikiutil.escape(d['title_text']))
 
+        html = '''
+<ul id="pagelocation">
+%s
+</ul>
+''' % "".join(content)
         return html
 
     def username(self, d):
@@ -177,29 +207,40 @@ class ThemeBase:
         """
         request = self.request
         _ = request.getText
-        preferencesPage = wikiutil.getSysPage(request, 'UserPreferences')
         
         userlinks = []
         # Add username/homepage link for registered users. We don't care
         # if it exists, the user can create it.
-        if request.user.valid:
-            homepage = Page(request, request.user.name)
-            title = homepage.split_title(request)
-            homelink = homepage.link_to(request, text=title)
+        if request.user.valid and request.user.name:
+            interwiki = wikiutil.getInterwikiHomePage(request)
+            name = request.user.name
+            aliasname = request.user.aliasname
+            if not aliasname:
+                aliasname = name
+            title = "%s @ %s" % (aliasname, interwiki[0])
+            # link to (interwiki) user homepage
+            homelink = (request.formatter.interwikilink(1, title=title, id="userhome", generated=True, *interwiki) +
+                        request.formatter.text(name) +
+                        request.formatter.interwikilink(0, title=title, id="userhome", *interwiki))
             userlinks.append(homelink)        
-            # Set pref page to localized Preferences page
-            title = preferencesPage.split_title(request)
-            userlinks.append(preferencesPage.link_to(request, text=title))
-        else:
-            # Add prefpage links with title: Login
-            userlinks.append(preferencesPage.link_to(request, text=_("Login")))
-            
-        userlinks = [u'<li>%s</li>\n' % link for link in userlinks]
-        html = u'<ul id="username">\n%s</ul>' % ''.join(userlinks)
+            # link to userprefs action
+            userlinks.append(d['page'].link_to(request, text=_('Preferences', formatted=False),
+                                               querystr={'action': 'userprefs'}, id="userprefs"))
+           
+        if request.cfg.show_login:
+            if request.user.valid:
+                userlinks.append(d['page'].link_to(request, text=_('Logout', formatted=False),
+                                                   querystr={'action': 'logout', 'logout': 'logout'}, id="logout"))
+            else:
+                userlinks.append(d['page'].link_to(request, text=_("Login", formatted=False),
+                                                   querystr={'action': 'login'}, id="login"))
+
+        userlinks = [u'<li>%s</li>' % link for link in userlinks]
+        html = u'<ul id="username">%s</ul>' % ''.join(userlinks)
         return html
 
-    # Schemas supported in toolbar links, using [url label] foramrt
-    linkSchemas = [r'http://', r'https://', r'ftp://', 'mailto:'] + \
+    # Schemas supported in toolbar links, using [url label] format
+    linkSchemas = [r'http://', r'https://', r'ftp://', 'mailto:', r'irc://', ] + \
                   [x + ':' for x in config.url_schemas]
 
     def splitNavilink(self, text, localize=1):
@@ -221,26 +262,11 @@ class ThemeBase:
             try:
                 pagename, title = text[1:-1].strip().split(' ', 1)
                 title = title.strip()
+                localize = 0
             except (ValueError, TypeError):
                 # Just use the text as is.
                 pagename = title = text
 
-            # Handle [url title] format
-            for scheme in self.linkSchemas:
-                if pagename.startswith(scheme):
-                    title = wikiutil.escape(title)
-                    link = '<a href="%s">%s</a>' % (pagename, title)
-                    break
-
-            # Handle [pagename title] format
-            else:
-                # Normalize page names, replace '_' with ' '. Usually
-                # all names use spaces internally, but for
-                # [name_with_spaces label] we must save the underscores
-                # until this point.
-                pagename = request.normalizePagename(pagename)
-                link = Page(request, pagename).link_to(request, title)
-
         # Handle regular pagename like "FrontPage"
         else:
             # Use localized pages for the current user
@@ -253,6 +279,42 @@ class ThemeBase:
             title = self.shortenPagename(title)
             link = page.link_to(request, title)
 
+
+        from MoinMoin import config
+        for scheme in self.linkSchemas:
+            if pagename.startswith(scheme):
+                title = wikiutil.escape(title)
+                link = '<a href="%s">%s</a>' % (pagename, title)
+                return pagename, link
+
+        # remove wiki: url prefix
+        if pagename.startswith("wiki:"):
+            pagename = pagename[5:]
+
+        # try handling interwiki links
+        try:
+            interwiki, page = pagename.split(':', 1)
+            thiswiki = request.cfg.interwikiname
+            if interwiki == thiswiki:
+                pagename = page
+                title = page
+            else:
+                return (pagename,
+                        self.request.formatter.interwikilink(True, interwiki, page) +
+                        page +
+                        self.request.formatter.interwikilink(False, interwiki, page)
+                        )
+        
+        except ValueError:
+            pass
+                  
+        # Normalize page names, replace '_' with ' '. Usually
+        # all names use spaces internally, but for
+        # [name_with_spaces label] we must save the underscores
+        # until this point.
+        pagename = request.normalizePagename(pagename)
+        link = Page(request, pagename).link_to(request, title)
+
         return pagename, link
 
     def shortenPagename(self, name):
@@ -265,7 +327,7 @@ class ThemeBase:
         If you don't like to do this in your theme, or want to use
         different algorithm, override this method.
         
-        @param text: page name, unicode
+        @param name: page name, unicode
         @rtype: unicode
         @return: shortened version.
         """
@@ -273,7 +335,7 @@ class ThemeBase:
         # First use only the sub page name, that might be enough
         if len(name) > maxLength:
             name = name.split('/')[-1]
-            # If its not enough, replace the middle with '...'
+            # If it's not enough, replace the middle with '...'
             if len(name) > maxLength:
                 half, left = divmod(maxLength - 3, 2)
                 name = u'%s...%s' % (name[:half + left], name[-half:])
@@ -300,21 +362,23 @@ class ThemeBase:
         if request.cfg.navi_bar:
             for text in request.cfg.navi_bar:
                 pagename, link = self.splitNavilink(text)
-                cls = 'wikilink'
                 if pagename == current:
                     cls = 'wikilink current'
+                else:
+                    cls = 'wikilink'
                 items.append(item % (cls, link))
                 found[pagename] = 1
 
         # Add user links to wiki links, eliminating duplicates.
         userlinks = request.user.getQuickLinks()
         for text in userlinks:
-            # Split text without localization, user know what she wants
+            # Split text without localization, user knows what he wants
             pagename, link = self.splitNavilink(text, localize=0)
             if not pagename in found:
-                cls = 'userlink'
                 if pagename == current:
                     cls = 'userlink current'
+                else:
+                    cls = 'userlink'
                 items.append(item % (cls, link))
                 found[pagename] = 1
 
@@ -327,14 +391,14 @@ class ThemeBase:
             items.append(item % (cls, link))
 
         # Assemble html
-        items = u'\n'.join(items)
+        items = u''.join(items)
         html = u'''
 <ul id="navibar">
 %s
 </ul>
 ''' % items
         return html
-                
+
     def get_icon(self, icon):
         """ Return icon data from self.icons
 
@@ -408,7 +472,7 @@ class ThemeBase:
         d['title'] = title % d
         d['i18ntitle'] = self.request.getText(d['title'], formatted=False)
         img_src = self.make_icon(icon, d)
-        return wikiutil.link_tag(self.request, page_params % d, img_src, attrs='title="%(i18ntitle)s"' % d)
+        return wikiutil.link_tag(self.request, page_params % d, img_src, title="%(i18ntitle)s" % d)
 
     def msg(self, d):
         """ Assemble the msg display
@@ -445,12 +509,28 @@ class ThemeBase:
         """
         request = self.request
         user = request.user
+        html = ''
         if user.valid and user.show_page_trail:
             trail = user.getTrail()
             if trail:
                 items = []
-                # Show all items except the last one which is this page.
-                for pagename in trail[:-1]:
+                for pagename in trail:
+                    try:
+                        interwiki, page = pagename.split(":", 1)
+                        # Items in trail are saved as valid interwiki
+                        # links, using _ for spaces.
+                        page = page.replace('_', ' ')
+                        if request.cfg.interwikiname != interwiki:
+                            link = (self.request.formatter.interwikilink(True, interwiki, page) +
+                                    self.shortenPagename(page) +
+                                    self.request.formatter.interwikilink(False, interwiki, page))
+                            items.append('<li>%s</li>' % link)
+                            continue
+                        else:
+                            pagename = page
+                            
+                    except ValueError:
+                        pass
                     page = Page(request, pagename)
                     title = page.split_title(request)
                     title = self.shortenPagename(title)
@@ -459,9 +539,8 @@ class ThemeBase:
                 html = '''
 <ul id="pagetrail">
 %s
-</ul>''' % '\n'.join(items)
-                return html
-        return ''
+</ul>''' % ''.join(items)
+        return html
 
     def html_stylesheets(self, d):
         """ Assemble html head stylesheet links
@@ -470,8 +549,7 @@ class ThemeBase:
         @rtype: string
         @return: stylesheets links
         """
-        link = ('<link rel="stylesheet" type="text/css" charset="%s"'
-                ' media="%s" href="%s">')
+        link = '<link rel="stylesheet" type="text/css" charset="%s" media="%s" href="%s">'
 
         # Check mode
         if d.get('print_mode'):
@@ -484,14 +562,37 @@ class ThemeBase:
         # Create stylesheets links
         html = []
         prefix = self.cfg.url_prefix
+        csshref = '%s/%s/css' % (prefix, self.name)
         for media, basename in stylesheets:
-            href = '%s/%s/css/%s.css' % (prefix, self.name, basename)
+            href = '%s/%s.css' % (csshref, basename)
             html.append(link % (self.stylesheetsCharset, media, href))
 
             # Don't add user css url if it matches one of ours
             if usercss and usercss == href:
                 usercss = None
 
+        # admin configurable additional css (farm or wiki level)
+        for media, csshref in self.request.cfg.stylesheets:
+            html.append(link % (self.stylesheetsCharset, media, csshref))
+
+        
+        # tribute to the most sucking browser: IE6
+        if self.cfg.hacks.get('ie7', False):
+            html.append("""
+<!-- compliance patch for microsoft browsers -->
+<!--[if lt IE 7]>
+   <script src="%s/common/ie7/ie7-standard-p.js" type="text/javascript"></script>
+<![endif]-->
+""" % prefix)
+
+        csshref = '%s/%s/css/msie.css' % (prefix, self.name)
+        html.append("""
+<!-- css only for MSIE browsers -->
+<!--[if IE]>
+   %s
+<![endif]-->
+""" % link % (self.stylesheetsCharset, 'all', csshref))
+
         # Add user css url (assuming that user css uses same charset)
         if usercss and usercss.lower() != "none":
             html.append(link % (self.stylesheetsCharset, 'all', usercss))
@@ -523,16 +624,16 @@ class ThemeBase:
     def pageinfo(self, page):
         """ Return html fragment with page meta data
 
-        Since page information use translated text, it use the ui
+        Since page information uses translated text, it uses the ui
         language and direction. It looks strange sometimes, but
-        translated text using page direction look worse.
+        translated text using page direction looks worse.
         
         @param page: current page
         @rtype: unicode
         @return: page last edit information
         """
         _ = self.request.getText
-        
+        html = ''
         if self.shouldShowPageinfo(page):
             info = page.lastEditInfo()
             if info:
@@ -540,11 +641,15 @@ class ThemeBase:
                     info = _("last edited %(time)s by %(editor)s") % info
                 else:
                     info = _("last modified %(time)s") % info
-                return '<p id="pageinfo" class="info"%(lang)s>%(info)s</p>\n' % {
+                pagename = page.page_name
+                if self.request.cfg.show_interwiki:
+                    pagename = "%s: %s" % (self.request.cfg.interwikiname, pagename)
+                info = "%s  (%s)" % (wikiutil.escape(pagename), info)
+                html = '<p id="pageinfo" class="info"%(lang)s>%(info)s</p>\n' % {
                     'lang': self.ui_lang_attr(),
                     'info': info
                     }
-        return ''
+        return html
     
     def searchform(self, d):
         """
@@ -601,10 +706,10 @@ searchBlur(e);
         """
         html = ''
         if self.cfg.show_version and not keywords.get('print_mode', 0):
-            html = (u'<div id="version">MoinMoin %s, Copyright 2000-2004 by '
-                    'Juergen Hermann</div>') % (version.revision,)
+            html = (u'<div id="version">MoinMoin Release %s [Revision %s], '
+                     'Copyright 2000-2006 by Juergen Hermann</div>') % (version.release, version.revision, )
         return html
-
+    
     def headscript(self, d):
         """ Return html head script with common functions
 
@@ -666,7 +771,7 @@ function searchBlur(e) {
 }
 
 function actionsMenuInit(title) {
-    // Initiliaze action menu
+    // Initialize action menu
     for (i = 0; i < document.forms.length; i++) {
         var form = document.forms[i];
         if (form.className == 'actionsmenu') {
@@ -692,35 +797,29 @@ function actionsMenuInit(title) {
 //-->
 </script>
 """ % {
-    'search_hint' : _('Search:', formatted=False).replace(':',''), # XXX TODO make own i18n string in 1.4
+    'search_hint' : _('Search', formatted=False),
     }
         return script
-    
-    def shouldUseRSS(self):
-        """ Return True if rss feature is available, or False
 
-        Currently rss is broken on plain Python, and works only when
-        installing PyXML. Return true if PyXML is installed.
+    def shouldUseRSS(self, page):
+        """ Return True if RSS feature is available and we are on the
+            RecentChanges page, or False.
         """
-        # Stolen from wikitest.py
-        try:
-            import xml
-            return xml.__file__.find('_xmlplus') != -1
-        except ImportError:
-            # This error reported on Python 2.2
+        if not rss_supported:
             return False
-        
-    def rsshref(self):
+        return page.page_name == u'RecentChanges' or \
+           page.page_name == self.request.getText(u'RecentChanges', formatted=False)
+
+    def rsshref(self, page):
         """ Create rss href, used for rss button and head link
         
         @rtype: unicode
-        @return: html head
+        @return: rss href
         """
-        href = u'%s/RecentChanges?action=rss_rc&amp;ddiffs=1&amp;unique=1' % \
-               self.request.getScriptname()
-        return href
-                           
-    def rsslink(self):
+        return (u'%s/RecentChanges?action=rss_rc&amp;ddiffs=1&amp;unique=1' 
+                % self.request.getScriptname())
+            
+    def rsslink(self, d):
         """ Create rss link in head, used by FireFox
 
         RSS link for FireFox. This shows an rss link in the bottom of
@@ -729,15 +828,14 @@ function actionsMenuInit(title) {
         @rtype: unicode
         @return: html head
         """
-        if self.shouldUseRSS():
-            link = [
-                u'<link rel="alternate"',
-                u' title="%s Recent Changes"' % self.cfg.sitename,
-                u' href="%s"' % self.rsshref(),
-                u' type="application/rss+xml">',
-                ]
-            return ''.join(link)
-        return ''
+        link = u''
+        page = d['page']
+        if self.shouldUseRSS(page):
+            link = (u'<link rel="alternate" title="%s Recent Changes" '
+                    u'href="%s" type="application/rss+xml">') % (
+                        self.cfg.sitename,
+                        self.rsshref(page) )
+        return link
        
     def html_head(self, d):
         """ Assemble html head
@@ -748,78 +846,43 @@ function actionsMenuInit(title) {
         """
         html = [
             u'<title>%(title)s - %(sitename)s</title>' % d,
+            self.externalScript('common'),
             self.headscript(d), # Should move to separate .js file
+            self.guiEditorScript(d),
             self.html_stylesheets(d),
-            self.rsslink(),
+            self.rsslink(d),
             ]
         return '\n'.join(html)
 
+    def externalScript(self, name):
+        """ Format external script html """
+        src = '%s/common/js/%s.js' % (self.request.cfg.url_prefix, name)
+        return '<script type="text/javascript" src="%s"></script>' % src
+
     def credits(self, d, **keywords):
         """ Create credits html from credits list """
         if isinstance(self.cfg.page_credits, (list, tuple)):
             items = ['<li>%s</li>' % i for i in self.cfg.page_credits]
-            html = '<ul id="credits">\n%s\n</ul>\n' % '\n'.join(items)
+            html = '<ul id="credits">\n%s\n</ul>\n' % ''.join(items)
         else:
             # Old config using string, output as is
             html = self.cfg.page_credits
         return html
 
-    def shouldShowEditbar(self, page):
-        """ Should we show the editbar?
-
-        Actions should implement this, because only the action knows if
-        the edit bar make sense. Until it goes into actions, we do the
-        checking here.
-
-        @param page: current page
-        @rtype: bool
-        @return: true if editbar should show
-        """
-        # Show editbar only for existing pages, including deleted pages,
-        # that the user may read. If you may not read, you can't edit,
-        # so you don't need editbar.
-        if (page.exists(includeDeleted=1) and
-            self.request.user.may.read(page.page_name)):
-            form = self.request.form
-            action = form.get('action', [''])[0]
-            # Do not show editbar on edit or edit preview
-            return not (action == 'edit' or form.has_key('button_preview'))
-        
-        return False
-
-    def subscribeLink(self, page):
-        """ Return subscribe/unsubscribe link to valid users
-        
-        @rtype: unicode
-        @return: subscribe or unsubscribe link
-        """
-        _ = self.request.getText
-        user = self.request.user
-        if self.cfg.mail_smarthost and user.valid:
-            # Email enabled and user valid, get current page status
-            if user.isSubscribedTo([page.page_name]):
-                title = _("Unsubscribe")
-            else:
-                title = _("Subscribe")
-            quotedname = wikiutil.quoteWikinameURL(page.page_name)
-            link = wikiutil.link_tag(self.request, quotedname + 
-                                     '?action=subscribe', title)
-            return link
-        return ''
-
     def actionsMenu(self, page):
         """ Create actions menu list and items data dict
         
-        The menu will contain the same items always, but items that are not 
-        available will be disabled (some broken browsers will let you select
-        disabled options though).
+        The menu will contain the same items always, but items that are
+        not available will be disabled (some broken browsers will let
+        you select disabled options though).
 
-        The menu should give best user experience for javascript enabled
-        browsers, and acceptable behavior for those who prefer not to
-        use Javascript.
+        The menu should give best user experience for javascript
+        enabled browsers, and acceptable behavior for those who prefer
+        not to use Javascript.
 
-        TODO: Move actionsMenuInit() into body onload. This require that
-        the theme will render body, its currently done on wikiutil/page.
+        TODO: Move actionsMenuInit() into body onload. This require
+        that the theme will render body, its currently done on
+        wikiutil/page.
         
         @param page: current page, Page object
         @rtype: unicode
@@ -831,30 +894,41 @@ function actionsMenuInit(title) {
         menu = [
             'raw',
             'print',
+            'RenderAsDocbook',
             'refresh',
             '__separator__',
-            'AttachFile',
             'SpellCheck',
             'LikePages',
             'LocalSiteMap',
             '__separator__',
             'RenamePage',
             'DeletePage',
+            '__separator__',
+            'MyPages',
+            'SubscribeUser',
+            '__separator__',
+            'Despam',
+            'PackagePages',
             ]
 
         titles = {
             # action: menu title
             '__title__': _("More Actions:", formatted=False),
-            '__separator__': '--------', # spacer
-            'raw': _('Show Raw Text', formatted=False),
-            'print': _('Show Print View', formatted=False),
+            # Translation may need longer or shorter separator
+            '__separator__': _('------------', formatted=False),
+            'raw': _('Raw Text', formatted=False),
+            'print': _('Print View', formatted=False),
             'refresh': _('Delete Cache', formatted=False),
-            'AttachFile': _('Attach File', formatted=False),
             'SpellCheck': _('Check Spelling', formatted=False), # rename action!
             'RenamePage': _('Rename Page', formatted=False),
             'DeletePage': _('Delete Page', formatted=False),
-            'LikePages': _('Show Like Pages', formatted=False),
-            'LocalSiteMap': _('Show Local Site Map', formatted=False),
+            'LikePages': _('Like Pages', formatted=False),
+            'LocalSiteMap': _('Local Site Map', formatted=False),
+            'MyPages': _('My Pages', formatted=False),
+            'SubscribeUser': _('Subscribe User', formatted=False),
+            'Despam': _('Remove Spam', formatted=False),
+            'PackagePages': _('Package Pages', formatted=False),
+            'RenderAsDocbook': _('Render as Docbook', formatted=False),
             }
 
         options = []
@@ -887,8 +961,9 @@ function actionsMenuInit(title) {
 
             options.append(option % data)
 
-        # Add custom actions not in the standard menu
-        more = [item for item in available if not item in titles]
+        # Add custom actions not in the standard menu, except for
+        # some actions like AttachFile (we have them on top level)
+        more = [item for item in available if not item in titles and not item in ('AttachFile',)]
         more.sort()
         if more:
             # Add separator
@@ -938,9 +1013,8 @@ actionsMenuInit('%(label)s');
     def editbar(self, d):
         """ Assemble the page edit bar.
 
-        Display on existing page. Replace iconbar, showtext, edit text,
-        refresh cache and available actions.
-        
+        Create html on first call, then return cached html.
+                
         @param d: parameter dictionary
         @rtype: unicode
         @return: iconbar html
@@ -949,46 +1023,176 @@ actionsMenuInit('%(label)s');
         if not self.shouldShowEditbar(page):
             return ''
 
-        # Use cached editbar if possible.
-        cacheKey = 'editbar'
-        cached = self._cache.get(cacheKey)
-        if cached:
-            return cached
-
-        # Make new edit bar
-        request = self.request
-        _ = self.request.getText
-        link = wikiutil.link_tag
-        quotedname = wikiutil.quoteWikinameURL(page.page_name)
-        links = []
-        add = links.append
+        html = self._cache.get('editbar')
+        if html is None:
+            # Remove empty items and format as list
+            items = ''.join(['<li>%s</li>' % item
+                             for item in self.editbarItems(page) if item])
+            html = u'<ul class="editbar">%s</ul>\n' % items
+            self._cache['editbar'] = html
         
-        # Parent page
-        parent = page.getParentPage()
-        if parent:
-           add(parent.link_to(request, _("Show Parent", formatted=False))) 
-        
-        # Page actions
-        if page.isWritable() and request.user.may.write(page.page_name):
-            add(link(request, quotedname + '?action=edit', _('Edit')))
-        else:
-            add(_('Immutable Page', formatted=False))              
-        
-        add(link(request, quotedname + '?action=diff',
-                 _('Show Changes', formatted=False)))
-        add(link(request, quotedname + '?action=info',
-                 _('Get Info', formatted=False)))
-        add(self.subscribeLink(page))
-        add(self.actionsMenu(page))
-        
-        # Format
-        items = '\n'.join(['<li>%s</li>' % item for item in links if item != ''])
-        html = u'<ul class="editbar">\n%s\n</ul>\n' % items
-        
-        # cache for next call
-        self._cache[cacheKey] = html
         return html
 
+    def shouldShowEditbar(self, page):
+        """ Should we show the editbar?
+
+        Actions should implement this, because only the action knows if
+        the edit bar makes sense. Until it goes into actions, we do the
+        checking here.
+
+        @param page: current page
+        @rtype: bool
+        @return: true if editbar should show
+        """
+        # Show editbar only for existing pages, including deleted pages,
+        # that the user may read. If you may not read, you can't edit,
+        # so you don't need editbar.
+        if (page.exists(includeDeleted=1) and
+            self.request.user.may.read(page.page_name)):
+            form = self.request.form
+            action = form.get('action', [''])[0]
+            # Do not show editbar on edit but on save/cancel
+            return not (action == 'edit' and
+                        not form.has_key('button_save') and
+                        not form.has_key('button_cancel'))
+        return False
+
+    def editbarItems(self, page):
+        """ Return list of items to show on the editbar 
+
+        This is separate method to make it easy to customize the
+        edtibar in sub classes.
+        """
+        return [self.editorLink(page),
+                self.infoLink(page),
+                self.subscribeLink(page),
+                self.quicklinkLink(page),
+                self.attachmentsLink(page),
+                self.actionsMenu(page),]
+
+    def guiworks(self, page):
+        """ Return whether the gui editor / converter can work for that page.
+
+            The GUI editor currently only works for wiki format.
+        """
+        return page.pi_format == 'wiki'
+        
+    def editorLink(self, page):
+        """ Return a link to the editor 
+        
+        If the user can't edit, return a disabled edit link.
+        
+        If the user want to show both editors, it will display "Edit
+        (Text)", otherwise as "Edit".
+        """
+        if not (page.isWritable() and
+                self.request.user.may.write(page.page_name)):
+            return self.disabledEdit()
+        
+        _ = self.request.getText
+        params = (wikiutil.quoteWikinameURL(page.page_name) +
+                  '?action=edit&amp;editor=')
+        
+        guiworks = self.guiworks(page)
+        if self.showBothEditLinks() and guiworks:
+            text = _('Edit (Text)', formatted=False)
+            params = params + 'text'
+            attrs = {'name': "texteditlink"}
+        else:
+            text = _('Edit', formatted=False)
+            if guiworks:
+                # 'textonly' will be upgraded dynamically to 'guipossible' by JS
+                params = params + 'textonly'
+                attrs = {'name': "editlink"}
+            else:
+                params = params + 'text'
+                attrs = {'name': "texteditlink"}
+        
+        return wikiutil.link_tag(self.request, params, text, **attrs)
+
+    def showBothEditLinks(self):
+        """ Return True if both edit links should be displayed """
+        editor = self.request.user.editor_ui
+        if editor == '<default>':
+            editor = self.request.cfg.editor_ui
+        return editor == 'freechoice'
+
+    def guiEditorScript(self, d):
+        """ Return a script that set the gui editor link variables
+        
+        The link will be created only when javascript is enabled and
+        the browser is compatible with the editor.
+        """
+        page = d['page']
+        if not (page.isWritable() and
+                self.request.user.may.write(page.page_name) and
+                self.showBothEditLinks() and
+                self.guiworks(page)):
+            return ''
+
+        _ = self.request.getText
+        return """\
+<script type="text/javascript">
+var gui_editor_link_href = "%(url)s?action=edit&editor=gui";
+var gui_editor_link_text = "%(text)s";
+</script>        
+""" % {'url': page.url(self.request),
+       'text': _('Edit (GUI)', formatted=False),}
+
+    def disabledEdit(self):
+        """ Return a disabled edit link """
+        _ = self.request.getText
+        return ('<span class="disabled">%s</span>' 
+                % _('Immutable Page', formatted=False))
+        
+    def infoLink(self, page):
+        """ Return link to page information """
+        _ = self.request.getText
+        return page.link_to(self.request,
+                            text=_('Info', formatted=False), 
+                            querystr='action=info')
+    
+    def subscribeLink(self, page):
+        """ Return subscribe/unsubscribe link to valid users
+        
+        @rtype: unicode
+        @return: subscribe or unsubscribe link
+        """
+        if not (self.cfg.mail_enabled and self.request.user.valid):
+            return ''
+        
+        _ = self.request.getText
+        if self.request.user.isSubscribedTo([page.page_name]):
+            text = _("Unsubscribe", formatted=False)
+        else:
+            text = _("Subscribe", formatted=False)
+        params = wikiutil.quoteWikinameURL(page.page_name) + '?action=subscribe'
+        return wikiutil.link_tag(self.request, params, text)
+
+    def quicklinkLink(self, page):
+        """ Return add/remove quicklink link
+        
+        @rtype: unicode
+        @return: link to add or remove a quicklink
+        """
+        if not self.request.user.valid:
+            return ''
+        
+        _ = self.request.getText
+        if self.request.user.isQuickLinkedTo([page.page_name]):
+            text = _("Remove Link", formatted=False)
+        else:
+            text = _("Add Link", formatted=False)
+        params = wikiutil.quoteWikinameURL(page.page_name) + '?action=quicklink'
+        return wikiutil.link_tag(self.request, params, text)
+
+    def attachmentsLink(self, page):
+        """ Return link to page attachments """
+        _ = self.request.getText
+        return page.link_to(self.request,
+                            text=_('Attachments', formatted=False), 
+                            querystr='action=AttachFile')
+
     def startPage(self):
         """ Start page div with page language and direction
         
@@ -1018,6 +1222,8 @@ actionsMenuInit('%(label)s');
         @return: page header html
         """
         return self.startPage()
+    
+    editorheader = header
         
     def footer(self, d, **keywords):
         """ Assemble page footer
@@ -1109,19 +1315,17 @@ actionsMenuInit('%(label)s');
         # Should use user interface language and direction
         html = '<div class="recentchanges"%s>\n' % self.ui_lang_attr()
         html += '<div>\n'
-        if self.shouldUseRSS():
+        page = d['page']
+        if self.shouldUseRSS(page):
             link = [
                 u'<div class="rcrss">',
-                u'<a href="%s">' % self.rsshref(),
+                u'<a href="%s">' % self.rsshref(page),
                 self.make_icon("rss"),
                 u'</a>',
                 u'</div>',
                 ]
             html += ''.join(link)
         html += '<p>'
-        if d['rc_update_bookmark']:
-            html += "%(rc_update_bookmark)s %(rc_curr_bookmark)s<br>" % d
-
         # Add day selector
         if d['rc_days']:
             days = []
@@ -1135,6 +1339,10 @@ actionsMenuInit('%(label)s');
                             str(day)))
             days = ' | '.join(days)
             html += (_("Show %s days.") % (days,))
+        
+        if d['rc_update_bookmark']:
+            html += " %(rc_update_bookmark)s %(rc_curr_bookmark)s" % d
+
         html += '</p>\n</div>\n'
 
         html += '<table>\n'
