@@ -12,7 +12,7 @@
     @license: GNU GPL, see COPYING for details.
 """
 
-import re, time
+import re, time, os
 from MoinMoin import action, config, macro, util
 from MoinMoin import wikiutil, wikiaction, i18n
 from MoinMoin.Page import Page
@@ -61,6 +61,7 @@ class Macro:
      * External macros - implemented in either MoinMoin.macro package, or
        in the specific wiki instance in the plugin/macro directory
     """
+    defaultDependency = ["time"]
 
     Dependencies = {
         "TitleSearch" : ["namespace"],
@@ -94,24 +95,31 @@ class Macro:
         self.formatter = self.request.formatter
         self._ = self.request.getText
         self.cfg = self.request.cfg
+        
+        # Initialized on execute
+        self.name = None
 
     def execute(self, macro_name, args):
-        macro = wikiutil.importPlugin(self.request.cfg, 'macro', macro_name)
-        if macro:
-            return macro(self, args)
+        """ Get and execute a macro 
+        
+        Try to get a plugin macro, or a builtin macro or a language
+        macro, or just raise ImportError. 
+        """
+        self.name = macro_name
+        try:
+            execute = wikiutil.importPlugin(self.cfg, 'macro', macro_name)
+        except wikiutil.PluginMissingError:
+            try:
+                builtins = self.__class__
+                execute = getattr(builtins, '_macro_' + macro_name)
+            except AttributeError:
+                if macro_name in i18n.languages:
+                    execute = builtins._m_lang
+                else:
+                    raise ImportError("Cannot load macro %s" % macro_name)
+        return execute(self, args)
 
-        builtins = vars(self.__class__)
-        # builtin macro
-        if builtins.has_key('_macro_' + macro_name):
-            return builtins['_macro_' + macro_name](self, args)
-
-        # language pseudo macro
-        if i18n.languages.has_key(macro_name):
-            return self._m_lang(macro_name, args)
-
-        raise ImportError("Cannot load macro %s" % macro_name)
-
-    def _m_lang(self, lang_name, text):
+    def _m_lang(self, text):
         """ Set the current language for page content.
         
             Language macro are used in two ways:
@@ -119,22 +127,21 @@ class Macro:
              * [lang(text)] - insert text with specific lang inside page
         """
         if text:
-            return (self.formatter.lang(1, lang_name) +
+            return (self.formatter.lang(1, self.name) +
                     self.formatter.text(text) +
-                    self.formatter.lang(0, lang_name))
+                    self.formatter.lang(0, self.name))
         
-        self.request.current_lang = lang_name
+        self.request.current_lang = self.name
         return ''
   
     def get_dependencies(self, macro_name):
-        if self.Dependencies.has_key(macro_name):
+        if macro_name in self.Dependencies:
             return self.Dependencies[macro_name]
-        result = wikiutil.importPlugin(self.request.cfg, 'macro', macro_name,
-                                       'Dependencies')
-        if result != None:
-            return result
-        else:
-            return ["time"]
+        try:
+            return wikiutil.importPlugin(self.request.cfg, 'macro',
+                                         macro_name, 'Dependencies')
+        except wikiutil.PluginError:
+            return self.defaultDependency
 
     def _macro_TitleSearch(self, args):
         return self._m_search("titlesearch")
@@ -198,7 +205,8 @@ class Macro:
         html = [
             u'<form method="get" action="">',
             u'<div>',
-            u'<input type="text" name="goto" size="30">',
+            u'<input type="hidden" name="action" value="goto">',
+            u'<input type="text" name="target" size="30">',
             u'<input type="submit" value="%s">' % _("Go To Page"),
             u'</div>',
             u'</form>',
@@ -207,9 +215,17 @@ class Macro:
         return self.formatter.rawHTML(html)
 
     def _macro_WordIndex(self, args):
-        s = ''
+        _ = self._
+        allpages = int(self.form.get('allpages', [0])[0]) != 0
         # Get page list readable by current user
-        pages = self.request.rootpage.getPageList()
+        # Filter by isSystemPage if needed
+        if allpages:
+            # TODO: make this fast by caching full page list
+            pages = self.request.rootpage.getPageList()
+        else:
+            def filter(name):
+                return not wikiutil.isSystemPage(self.request, name)
+            pages = self.request.rootpage.getPageList(filter=filter)
         map = {}
         word_re = re.compile(u'[%s][%s]+' % (config.chars_upper, config.chars_lower), re.UNICODE)
         for name in pages:
@@ -223,14 +239,15 @@ class Macro:
         all_words = map.keys()
         all_words.sort()
         index_letters = []
-        last_letter = None
+        current_letter = None
         html = []
         for word in all_words:
-            letter = word[0]
-            if letter != last_letter:
+            letter = wikiutil.getUnicodeIndexGroup(word)
+            if letter != current_letter:
                 #html.append(self.formatter.anchordef()) # XXX no text param available!
-                html.append(u'<a name="%s">\n<h3>%s</h3>\n' % (wikiutil.quoteWikinameURL(letter), letter))
-                last_letter = letter
+                html.append(u'<a name="%s"><h3>%s</h3></a>' % (
+                    wikiutil.quoteWikinameURL(letter), letter.replace('~', 'Others')))
+                current_letter = letter
             if letter not in index_letters:
                 index_letters.append(letter)
 
@@ -242,12 +259,18 @@ class Macro:
             links.sort()
             last_page = None
             for name in links:
-                if name == last_page: continue
+                if name == last_page:
+                    continue
                 html.append(self.formatter.listitem(1))
                 html.append(Page(self.request, name).link_to(self.request))
                 html.append(self.formatter.listitem(0))
             html.append(self.formatter.bullet_list(0))
-        return u'%s%s' % (_make_index_key(index_letters), u''.join(html))
+        
+        qpagename = wikiutil.quoteWikinameURL(self.formatter.page.page_name)
+        index = _make_index_key(index_letters, u"""<br>
+<a href="%s?allpages=%d">%s</a>
+""" % (qpagename, not allpages, (_('Include system pages'), _('Exclude system pages'))[allpages]) )
+        return u'%s%s' % (index, u''.join(html)) 
 
 
     def _macro_TitleIndex(self, args):
@@ -272,7 +295,6 @@ class Macro:
                 
         current_letter = None
         for name in pages:
-            letter = name[0].upper()
             letter = wikiutil.getUnicodeIndexGroup(name)
             if letter not in index_letters:
                 index_letters.append(letter)
@@ -289,7 +311,7 @@ class Macro:
         if 0: # if wikixml.ok: # XXX currently switched off (not implemented)
             from MoinMoin import wikixml
             index = (index + self.formatter.url(1, 
-                wikiutil.quoteWikinameURL(self.formatter.page.page_name) + "?action=rss_ti", unescaped=1) +
+                wikiutil.quoteWikinameURL(self.formatter.page.page_name) + "?action=rss_ti", do_escape=0) +
                      self.formatter.icon("rss") +
                      self.formatter.url(0))
 
@@ -326,12 +348,34 @@ class Macro:
 
         return self.formatter.rawHTML(buf.getvalue())
 
-
     def _macro_SystemInfo(self, args):
         import operator, sys
         from StringIO import StringIO
         from MoinMoin import parser, processor, version
         from MoinMoin.logfile import editlog, eventlog
+        def _formatInReadableUnits(size):
+            size = float(size)
+            unit = u' Byte'
+            if size > 9999:
+                unit = u' KiB'
+                size /= 1024
+            if size > 9999:
+                unit = u' MiB'
+                size /= 1024
+            if size > 9999:
+                unit = u' GiB'
+                size /= 1024
+            return u"%.1f %s" % (size, unit)
+
+        def _getDirectorySize(path):
+            try:
+                dirsize = 0
+                for root, dirs, files in os.walk(path):
+                    dirsize += sum([os.path.getsize(os.path.join(root, name)) for name in files])
+            except EnvironmentError, e:
+                dirsize = -1
+            return dirsize
+
         _ = self._
         # check for 4XSLT
         try:
@@ -342,6 +386,13 @@ class Macro:
         except AttributeError:
             ftversion = 'N/A'
 
+        t_count = None
+        try:
+            from threading import activeCount
+            t_count = activeCount()
+        except ImportError:
+            pass
+
         # Get the full pagelist in the wiki
         pagelist = self.request.rootpage.getPageList(user='')
         totalsize = reduce(operator.add, [Page(self.request, name).size()
@@ -349,50 +400,48 @@ class Macro:
 
         buf = StringIO()
         row = lambda label, value, buf=buf: buf.write(
-            u'<dt>%s</dt><dd>%s</dd>' %
-            (label, value))
+            u'<dt>%s</dt><dd>%s</dd>' % (label, value))
 
         buf.write(u'<dl>')
         row(_('Python Version'), sys.version)
         row(_('MoinMoin Version'), _('Release %s [Revision %s]') % (version.release, version.revision))
         if ftversion:
             row(_('4Suite Version'), ftversion)
-        row(_('Number of pages'), str(len(pagelist)))
         systemPages = [page for page in pagelist
                        if wikiutil.isSystemPage(self.request, page)]
+        row(_('Number of pages'), str(len(pagelist)-len(systemPages)))
         row(_('Number of system pages'), str(len(systemPages)))
-        row(_('Accumulated page sizes'), str(totalsize))
+        row(_('Accumulated page sizes'), _formatInReadableUnits(totalsize))
+        data_dir = self.request.cfg.data_dir
+        row(_('Disk usage of %(data_dir)s/pages/') % {'data_dir': data_dir},
+            _formatInReadableUnits(_getDirectorySize(os.path.join(data_dir, 'pages'))))
+        row(_('Disk usage of %(data_dir)s/') % {'data_dir': data_dir},
+            _formatInReadableUnits(_getDirectorySize(data_dir)))
 
         edlog = editlog.EditLog(self.request)
-        row(_('Entries in edit log'),
-            _("%(logcount)s (%(logsize)s bytes)") %
-            {'logcount': edlog.lines(), 'logsize': edlog.size()})
+        row(_('Entries in edit log'), "%s (%s)" % (edlog.lines(), _formatInReadableUnits(edlog.size())))
 
-        # !!! This puts a heavy load on the server when the log is large,
-        # and it can appear on normal pages ==> so disable it for now.
+        # This puts a heavy load on the server when the log is large
         eventlogger = eventlog.EventLog(self.request)
         nonestr = _("NONE")
-        row('Event log',
-            "%s bytes" % eventlogger.size())
-        row(_('Global extension macros'), 
-            ', '.join(macro.extension_macros) or nonestr)
+        row('Event log', _formatInReadableUnits(eventlogger.size()))
+        row(_('Global extension macros'), ', '.join(macro.extension_macros) or nonestr)
         row(_('Local extension macros'), 
             ', '.join(wikiutil.wikiPlugins('macro', self.cfg)) or nonestr)
-        ext_actions = []
-        for a in action.extension_actions:
-            if not a in self.request.cfg.excluded_actions:
-                ext_actions.append(a)
-        row(_('Global extension actions'), 
-            ', '.join(ext_actions) or nonestr)
+        ext_actions = [x for x in action.extension_actions
+                       if not x in self.request.cfg.actions_excluded]
+        row(_('Global extension actions'), ', '.join(ext_actions) or nonestr)
         row(_('Local extension actions'), 
             ', '.join(wikiaction.getPlugins(self.request)[1]) or nonestr)
-        row(_('Installed parsers'), 
-            ', '.join(parser.modules) or nonestr)
+        row(_('Global parsers'), ', '.join(parser.modules) or nonestr)
+        row(_('Local extension parsers'), 
+            ', '.join(wikiutil.wikiPlugins('parser', self.cfg)) or nonestr)
         row(_('Installed processors (DEPRECATED -- use Parsers instead)'), 
             ', '.join(processor.processors) or nonestr)
         state = (_('Disabled'), _('Enabled'))
         row(_('Lupy search'), state[self.request.cfg.lupy_search])
-        buf.write(u'</dl')
+        row(_('Active threads'), t_count or 'N/A')
+        buf.write(u'</dl>')
 
         return self.formatter.rawHTML(buf.getvalue())
 
@@ -409,7 +458,9 @@ class Macro:
         except KeyError:
             # Wrong argument, return inline error message
             arg = self.formatter.text(args)
-            return '<span class="error">Wrong argument: %s</span>' % arg
+            return (self.formatter.span(1, css_class="error") +
+                    'Wrong argument: %s' % arg +
+                    self.formatter.span(0))
         
         count = self.request.rootpage.getPageCount(exists=exists)
         return self.formatter.text("%d" % count)
@@ -514,7 +565,15 @@ class Macro:
 
     def _macro_UserPreferences(self, args):
         from MoinMoin import userform
-        return self.formatter.rawHTML(userform.getUserForm(self.request))
+
+        create_only = False
+        if isinstance(args, unicode):
+            args = args.strip(" '\"")
+            create_only = (args.lower()=="createonly")
+
+        return self.formatter.rawHTML(userform.getUserForm(
+            self.request,
+            create_only=create_only))
 
     def _macro_Anchor(self, args):
         return self.formatter.anchordef(args or "anchor")
@@ -534,9 +593,7 @@ class Macro:
         if self.request.user.valid:
             # decode address and generate mailto: link
             email = decodeSpamSafeEmail(email)
-            icon = util.web.getLinkIcon(self.request, self.formatter, "mailto")
-            result = (self.formatter.url(1, 'mailto:' + email, 'external', pretty_url=1, unescaped=1) +
-                      icon +
+            result = (self.formatter.url(1, 'mailto:' + email, css='mailto', do_escape=0) +
                       self.formatter.text(text or email) +
                       self.formatter.url(0))
         else:
@@ -552,7 +609,6 @@ class Macro:
 
         return result
 
-
     def _macro_GetVal(self, args):
         page,key = args.split(',')
         d = self.request.dicts.dict(page)
