@@ -6,11 +6,12 @@
     @license: GNU GPL, see COPYING for details.
 """
     
-import os, re, difflib, urllib
+import os, re, urllib, cgi, time
+import codecs
 
+from MoinMoin.support import difflib
 from MoinMoin import util, version, config
-from MoinMoin.util import pysupport
-
+from MoinMoin.util import pysupport, filesys
 
 # Exceptions
 class InvalidFileNameError(Exception):
@@ -38,7 +39,6 @@ def decodeWindowsPath(text):
     """
 
     import locale
-    import codecs
     cur_charset = locale.getdefaultlocale()[1]
     try:
         return unicode(text, 'utf-8')
@@ -83,7 +83,7 @@ def decodeUserInput(s, charsets=[config.charset]):
     Decodes input from the user.
     
     @param s: the string to unquote
-    @param charset: the charset to assume the string is in
+    @param charsets: list of charsets to assume the string is in
     @rtype: unicode
     @return: the unquoted string as unicode
     """
@@ -92,7 +92,107 @@ def decodeUserInput(s, charsets=[config.charset]):
             return s.decode(charset)
         except UnicodeError:
             pass
-    raise UnicodeError('The string "%s" cannot be decoded.' % s)
+    raise UnicodeError('The string %r cannot be decoded.' % s)
+
+
+# this is a thin wrapper around urllib (urllib only handles str, not unicode)
+# with py <= 2.4.1, it would give incorrect results with unicode
+# with py == 2.4.2, it crashes with unicode, if it contains non-ASCII chars
+def url_quote(s, safe='/', want_unicode=False):
+    """
+    Wrapper around urllib.quote doing the encoding/decoding as usually wanted:
+    
+    @param s: the string to quote (can be str or unicode, if it is unicode,
+              config.charset is used to encode it before calling urllib)
+    @param safe: just passed through to urllib
+    @param want_unicode: for the less usual case that you want to get back
+                         unicode and not str, set this to True
+                         Default is False.
+    """
+    if isinstance(s, unicode):
+        s = s.encode(config.charset)
+    elif not isinstance(s, str):
+        s = str(s)
+    s = urllib.quote(s, safe)
+    if want_unicode:
+        s = s.decode(config.charset) # ascii would also work
+    return s
+
+def url_quote_plus(s, safe='/', want_unicode=False):
+    """
+    Wrapper around urllib.quote_plus doing the encoding/decoding as usually wanted:
+    
+    @param s: the string to quote (can be str or unicode, if it is unicode,
+              config.charset is used to encode it before calling urllib)
+    @param safe: just passed through to urllib
+    @param want_unicode: for the less usual case that you want to get back
+                         unicode and not str, set this to True
+                         Default is False.
+    """
+    if isinstance(s, unicode):
+        s = s.encode(config.charset)
+    elif not isinstance(s, str):
+        s = str(s)
+    s = urllib.quote_plus(s, safe)
+    if want_unicode:
+        s = s.decode(config.charset) # ascii would also work
+    return s
+
+def url_unquote(s, want_unicode=True):
+    """
+    Wrapper around urllib.unquote doing the encoding/decoding as usually wanted:
+    
+    @param s: the string to unquote (can be str or unicode, if it is unicode,
+              config.charset is used to encode it before calling urllib)
+    @param want_unicode: for the less usual case that you want to get back
+                         str and not unicode, set this to False.
+                         Default is True.
+    """
+    if isinstance(s, unicode):
+        s = s.encode(config.charset) # ascii would also work
+    s = urllib.unquote(s)
+    if want_unicode:
+        s = s.decode(config.charset)
+    return s
+
+def parseQueryString(qstr, want_unicode=True):
+    """ Parse a querystring "key=value&..." into a dict.
+    """
+    is_unicode = isinstance(qstr, unicode)
+    if is_unicode:
+        qstr = qstr.encode(config.charset)
+    values = {}
+    for key, value in cgi.parse_qs(qstr).items():
+        if len(value) < 2:
+            v = ''.join(value)
+            if want_unicode:
+                try:
+                    v = unicode(v, config.charset)
+                except UnicodeDecodeError:
+                    v = unicode(v, 'iso-8859-1', 'replace')
+            values[key] = v
+    return values
+
+def makeQueryString(qstr=None, want_unicode=False, **kw):
+    """ Make a querystring from arguments.
+        
+    kw arguments overide values in qstr.
+
+    If a string is passed in, it's returned verbatim and
+    keyword parameters are ignored.
+
+    @param qstr: dict to format as query string, using either ascii or unicode
+    @param kw: same as dict when using keywords, using ascii or unicode
+    @rtype: string
+    @return: query string ready to use in a url
+    """
+    if qstr is None:
+        qstr = {}
+    if isinstance(qstr, type({})):
+        qstr.update(kw)
+        items = ['%s=%s' % (url_quote_plus(key, want_unicode=want_unicode), url_quote_plus(value, want_unicode=want_unicode)) for key, value in qstr.items()]
+        qstr = '&'.join(items)
+    return qstr
 
 
 # FIXME: better name would be quoteURL, as this is useful for any
@@ -103,9 +203,9 @@ def quoteWikinameURL(pagename, charset=config.charset):
     Use urllib.quote to quote any character that is not always safe. 
 
     @param pagename: the original pagename (unicode)
-    @charset: url text encoding, 'utf-8' recommended. Other charsert
-              might not be able to encode the page name and raise
-              UnicodeError. (default config.charset ('utf-8')).
+    @param charset: url text encoding, 'utf-8' recommended. Other charsert
+                    might not be able to encode the page name and raise
+                    UnicodeError. (default config.charset ('utf-8')).
     @rtype: string
     @return: the quoted filename, all unsafe characters encoded
     """
@@ -141,6 +241,38 @@ def escape(s, quote=0):
         s = s.replace('"', "&quot;")
     return s
 
+def clean_comment(comment):
+    """ Clean comment - replace CR, LF, TAB by whitespace, delete control chars
+        TODO: move this to config, create on first call then return cached.
+    """
+    # we only have input fields with max 200 chars, but spammers send us more
+    if len(comment) > 201:
+        comment = u''
+    remap_chars = {
+        ord(u'\t'): u' ',
+        ord(u'\r'): u' ',
+        ord(u'\n'): u' ',
+    }
+    control_chars = u'\x00\x01\x02\x03\x04\x05\x06\x07\x08\x0b\x0c\x0e\x0f' \
+                    '\x10\x11\x12\x13\x14\x15\x16\x17\x18\x19\x1a\x1b\x1c\x1d\x1e\x1f'
+    for c in control_chars:
+        remap_chars[c] = None
+    comment = comment.translate(remap_chars)
+    return comment
+
+def make_breakable(text, maxlen):
+    """ make a text breakable by inserting spaces into nonbreakable parts
+    """
+    text = text.split(" ")
+    newtext = []
+    for part in text:
+        if len(part) > maxlen:
+            while part:
+                newtext.append(part[:maxlen])
+                part = part[maxlen:]
+        else:
+            newtext.append(part)
+    return " ".join(newtext)
 
 ########################################################################
 ### Storage
@@ -203,7 +335,7 @@ def unquoteWikiname(filename, charsets=[config.charset]):
     Todo: search clients of unquoteWikiname and check for exceptions. 
 
     @param filename: string using charset and possibly quoted parts
-    @param charset: charset used by string
+    @param charsets: list of charsets used by string
     @rtype: Unicode String
     @return: WikiName
     """
@@ -265,7 +397,90 @@ def version2timestamp(v):
         This must ONLY be used for display purposes.
     """
     return v/1000000.0
+
+
+# This is the list of meta attribute names to be treated as integers.
+# IMPORTANT: do not use any meta attribute names with "-" (or any other chars
+# invalid in python attribute names), use e.g. _ instead.
+INTEGER_METAS = ['current', 'revision', # for page storage (moin 2.0)
+                 'data_format_revision', # for data_dir format spec (use by mig scripts)
+                ]
+
+class MetaDict(dict):
+    """ store meta informations as a dict """
+    def __init__(self, metafilename):
+        """ create a MetaDict from metafilename """
+        dict.__init__(self)
+        self.metafilename = metafilename
+        self.dirty = False
+        self.loaded = False
+
+    def _get_meta(self):
+        """ get the meta dict from an arbitrary filename.
+            does not keep state, does uncached, direct disk access.
+            @param metafilename: the name of the file to read
+            @return: dict with all values or {} if empty or error
+        """
+        # XXX what does happen if the metafile is being written to in another process?
+        try:
+            metafile = codecs.open(self.metafilename, "r", "utf-8")
+            meta = metafile.read() # this is much faster than the file's line-by-line iterator
+            metafile.close()
+        except IOError:
+            meta = u''
+        for line in meta.splitlines():
+            key, value = line.split(':', 1)
+            value = value.strip()
+            if key in INTEGER_METAS:
+                value = int(value)
+            dict.__setitem__(self, key, value)
+        self.loaded = True
     
+    def _put_meta(self):
+        """ put the meta dict into an arbitrary filename.
+            does not keep or modify state, does uncached, direct disk access.
+            @param metafilename: the name of the file to write
+            @param metadata: dict of the data to write to the file
+        """
+        meta = []
+        for key, value in self.items():
+            if key in INTEGER_METAS:
+                value = str(value)
+            meta.append("%s: %s" % (key, value))
+        meta = '\n'.join(meta)
+        # XXX what does happen if the metafile is being read or written to in another process?
+        metafile = codecs.open(self.metafilename, "w", "utf-8")
+        metafile.write(meta)
+        metafile.close()
+        self.dirty = False
+
+    def sync(self, mtime_usecs=None):
+        """ sync the in-memory dict to disk (if dirty) """
+        if self.dirty:
+            if not mtime_usecs is None:
+                self.__setitem__('mtime', str(mtime_usecs))
+            self._put_meta()
+
+    def __getitem__(self, key):
+        try:
+            return dict.__getitem__(self, key)
+        except KeyError:
+            if not self.loaded:
+                self._get_meta() # lazy loading of metadata
+                return dict.__getitem__(self, key)
+            else:
+                raise
+
+    def __setitem__(self, key, value):
+        try:
+            oldvalue = dict.__getitem__(self, key)
+        except KeyError:
+            oldvalue = None
+        if value != oldvalue:
+            dict.__setitem__(self, key, value)
+            self.dirty = True
+
+
 #############################################################################
 ### InterWiki
 #############################################################################
@@ -285,10 +500,8 @@ def split_wiki(wikiurl):
         try:
             wikitag, tail = wikiurl.split("/", 1)
         except ValueError:
-            wikitag = None
-            tail = None
-
-    return (wikitag, tail)
+            wikitag, tail = 'Self', wikiurl
+    return wikitag, tail
 
 
 def join_wiki(wikiurl, wikitail):
@@ -313,7 +526,7 @@ def resolve_wiki(request, wikiurl):
     @param request: the request object
     @param wikiurl: the InterWiki:PageName link
     @rtype: tuple
-    @return: (wikitag, wikiurl, wikitail)
+    @return: (wikitag, wikiurl, wikitail, err)
     """
     # load map (once, and only on demand)
     try:
@@ -360,7 +573,7 @@ def resolve_wiki(request, wikiurl):
     wikitag, tail = split_wiki(wikiurl)
 
     # return resolved url
-    if wikitag and _interwiki_list.has_key(wikitag):
+    if _interwiki_list.has_key(wikitag):
         return (wikitag, _interwiki_list[wikitag], tail, False)
     else:
         return (wikitag, request.getScriptname(), "/InterWiki", True)
@@ -379,8 +592,7 @@ def isSystemPage(request, pagename):
     @return: true if page is a system page
     """
     return (request.dicts.has_member('SystemPagesGroup', pagename) or
-        isTemplatePage(request, pagename) or
-        isFormPage(request, pagename))
+        isTemplatePage(request, pagename))
 
 
 def isTemplatePage(request, pagename):
@@ -394,16 +606,6 @@ def isTemplatePage(request, pagename):
     return filter.search(pagename) is not None
 
 
-def isFormPage(request, pagename):
-    """ Is this a form page?
-    
-    @param pagename: the page name
-    @rtype: bool
-    @return: true if page is a form page
-    """
-    filter = re.compile(request.cfg.page_form_regex, re.UNICODE)
-    return filter.search(pagename) is not None
-
 def isGroupPage(request, pagename):
     """ Is this a name of group page?
 
@@ -479,16 +681,17 @@ def getFrontPage(request):
 
     @param request: current request
     @rtype: Page object
-    @return localized FrontPage
+    @return localized page_front_page, if there is a translation
     """
     return getSysPage(request, request.cfg.page_front_page)
     
 
-# FIXME - move to user class?
 def getHomePage(request, username=None):
     """
     Get a user's homepage, or return None for anon users and
     those who have not created a homepage.
+
+    DEPRECATED - try to use getInterwikiHomePage (see below)
     
     @param request: the request object
     @param username: the user's name
@@ -510,6 +713,34 @@ def getHomePage(request, username=None):
     return None
 
 
+def getInterwikiHomePage(request, username=None):
+    """
+    Get a user's homepage.
+    
+    cfg.user_homewiki influences behaviour of this:
+    'Self' does mean we store user homepage in THIS wiki.
+    When set to our own interwikiname, it behaves like with 'Self'.
+    
+    'SomeOtherWiki' means we store user homepages in another wiki.
+    
+    @param request: the request object
+    @param username: the user's name
+    @rtype: tuple (or None for anon users)
+    @return: (wikiname, pagename)
+    """
+    # default to current user
+    if username is None and request.user.valid:
+        username = request.user.name
+    if not username:
+        return None # anon user
+
+    homewiki = request.cfg.user_homewiki
+    if homewiki == request.cfg.interwikiname:
+        homewiki = 'Self'
+
+    return homewiki, username
+
+
 def AbsPageName(request, context, pagename):
     """
     Return the absolute pagename for a (possibly) relative pagename.
@@ -519,17 +750,16 @@ def AbsPageName(request, context, pagename):
     @rtype: string
     @return: the absolute page name
     """
-    if config.allow_subpages:
-        if pagename.startswith(PARENT_PREFIX):
-            pagename = '/'.join(filter(None, context.split('/')[:-1] + [pagename[PARENT_PREFIX_LEN:]]))
-        elif pagename.startswith(CHILD_PREFIX):
-            pagename = context + '/' + pagename[CHILD_PREFIX_LEN:]
-
+    if pagename.startswith(PARENT_PREFIX):
+        pagename = '/'.join(filter(None, context.split('/')[:-1] + [pagename[PARENT_PREFIX_LEN:]]))
+    elif pagename.startswith(CHILD_PREFIX):
+        pagename = context + '/' + pagename[CHILD_PREFIX_LEN:]
     return pagename
 
 def pagelinkmarkup(pagename):
     """ return markup that can be used as link to page <pagename> """
-    if isStrictWikiname(pagename): # XXX TODO re.match(Parser.word_rule, pagename)
+    from MoinMoin.parser.wiki import Parser
+    if re.match(Parser.word_rule + "$", pagename):
         return pagename
     else:
         return u'["%s"]' % pagename
@@ -538,68 +768,77 @@ def pagelinkmarkup(pagename):
 ### Plugins
 #############################################################################
 
+class PluginError(Exception):
+    """ Base class for plugin errors """
+
+class PluginMissingError(PluginError):
+    """ Raised when a plugin is not found """
+
+class PluginAttributeError(PluginError):
+    """ Raised when plugin does not contain an attribtue """
+
+
 def importPlugin(cfg, kind, name, function="execute"):
     """ Import wiki or builtin plugin
     
-    Returns an object from a plugin module or None if module or
-    'function' is not found.
+    Returns function from a plugin module name. If name can not be
+    imported, raise PluginMissingError. If function is missing, raise
+    PluginAttributeError.
 
     kind may be one of 'action', 'formatter', 'macro', 'processor',
     'parser' or any other directory that exist in MoinMoin or
     data/plugin
 
     Wiki plugins will always override builtin plugins. If you want
-    specific plugin, use either importWikiPlugin or importName directly.
+    specific plugin, use either importWikiPlugin or importBuiltinPlugin
+    directly.
     
     @param cfg: wiki config instance
     @param kind: what kind of module we want to import
     @param name: the name of the module
     @param function: the function name
-    @rtype: callable
+    @rtype: any object
     @return: "function" of module "name" of kind "kind", or None
     """
-    # Try to import from the wiki
-    plugin = importWikiPlugin(cfg, kind, name, function)
-    if plugin is None:
-        # Try to get the plugin from MoinMoin
-        modulename = 'MoinMoin.%s.%s' % (kind, name)
-        plugin = pysupport.importName(modulename, function)
-        
-    return plugin
+    try:
+        return importWikiPlugin(cfg, kind, name, function)
+    except PluginMissingError:
+        return importBuiltinPlugin(kind, name, function)
 
-def importWikiPlugin(cfg, kind, name, function):
+
+def importWikiPlugin(cfg, kind, name, function="execute"):
     """ Import plugin from the wiki data directory
     
-    We try to import only ONCE - then cache the plugin, even if we got
-    None. This way we prevent expensive import of existing plugins for
-    each call to a plugin.
-
-    @param cfg: wiki config instance
-    @param kind: what kind of module we want to import
-    @param name: the name of the module
-    @param function: the function name
-    @rtype: callable
-    @return: "function" of module "name" of kind "kind", or None
+    See importPlugin docstring.
     """
+    if not name in wikiPlugins(kind, cfg):
+        raise PluginMissingError
+    moduleName = '%s.plugin.%s.%s' % (cfg.siteid, kind, name)
+    return importNameFromPlugin(moduleName, function)
 
-    # Wiki plugins are located under 'wikiconfigname.plugin' module.
-    modulename = '%s.plugin.%s.%s' % (cfg.siteid, kind, name)
-    key = (modulename, function)
+
+def importBuiltinPlugin(kind, name, function="execute"):
+    """ Import builtin plugin from MoinMoin package 
+    
+    See importPlugin docstring.
+    """
+    if not name in builtinPlugins(kind):
+        raise PluginMissingError
+    moduleName = 'MoinMoin.%s.%s' % (kind, name)
+    return importNameFromPlugin(moduleName, function)
+
+
+def importNameFromPlugin(moduleName, name):
+    """ Return name from plugin module 
+    
+    Raise PluginAttributeError if name does not exists.
+    """
+    module = __import__(moduleName, globals(), {}, [name])
     try:
-        # Try cache first - fast!
-        plugin = cfg._wiki_plugins[key]
-    except (KeyError, AttributeError):
-        # Try to import from disk and cache result - slow!
-        plugin = pysupport.importName(modulename, function)
-        try:
-            cfg._wiki_plugins[key] = plugin
-        except AttributeError:
-            cfg._wiki_plugins = {key: plugin}
-    return plugin
+        return getattr(module, name)
+    except AttributeError:
+        raise PluginAttributeError
 
-# If we use threads, make this function thread safe
-if config.use_threads:
-    importWikiPlugin = pysupport.makeThreadSafe(importWikiPlugin)
 
 def builtinPlugins(kind):
     """ Gets a list of modules in MoinMoin.'kind'
@@ -609,12 +848,14 @@ def builtinPlugins(kind):
     @return: module names
     """
     modulename = "MoinMoin." + kind
-    plugins = pysupport.importName(modulename, "modules")
-    return plugins or []
+    return pysupport.importName(modulename, "modules")
 
 
 def wikiPlugins(kind, cfg):
     """ Gets a list of modules in data/plugin/'kind'
+ 
+    Require valid plugin directory. e.g missing 'parser' directory or
+    missing '__init__.py' file will raise errors.
     
     @param kind: what kind of modules we look for
     @rtype: list
@@ -622,8 +863,7 @@ def wikiPlugins(kind, cfg):
     """
     # Wiki plugins are located in wikiconfig.plugin module
     modulename = '%s.plugin.%s' % (cfg.siteid, kind)
-    plugins = pysupport.importName(modulename, "modules")
-    return plugins or []
+    return pysupport.importName(modulename, "modules")
 
 
 def getPlugins(kind, cfg):
@@ -665,15 +905,17 @@ def getParserForExtension(cfg, extension):
         import types
         etp, etd = {}, None
         for pname in getPlugins('parser', cfg):
-            Parser = importPlugin(cfg, 'parser', pname, 'Parser')
-            if Parser is not None:
-                if hasattr(Parser, 'extensions'):
-                    exts = Parser.extensions
-                    if type(exts) == types.ListType:
-                        for ext in Parser.extensions:
-                            etp[ext] = Parser
-                    elif str(exts) == '*':
-                        etd = Parser
+            try:
+                Parser = importPlugin(cfg, 'parser', pname, 'Parser')
+            except PluginMissingError:
+                continue
+            if hasattr(Parser, 'extensions'):
+                exts = Parser.extensions
+                if type(exts) == types.ListType:
+                    for ext in Parser.extensions:
+                        etp[ext] = Parser
+                elif str(exts) == '*':
+                    etd = Parser
         cfg._EXT_TO_PARSER = etp
         cfg._EXT_TO_PARSER_DEFAULT = etd
         
@@ -689,8 +931,8 @@ def parseAttributes(request, attrstring, endtoken=None, extension=None):
     Parse a list of attributes and return a dict plus a possible
     error message.
     If extension is passed, it has to be a callable that returns
-    None when it was not interested into the token, '' when all was OK
-    and it did eat the token, and any other string to return an error
+    a tuple (found_flag, msg). found_flag is whether it did find and process
+    something, msg is '' when all was OK or any other string to return an error
     message.
     
     @param request: the request object
@@ -721,9 +963,13 @@ def parseAttributes(request, attrstring, endtoken=None, extension=None):
 
         # call extension function with the current token, the parser, and the dict
         if extension:
-            msg = extension(key, parser, attrs)
-            if msg == '': continue
-            if msg: break
+            found_flag, msg = extension(key, parser, attrs)
+            #request.log("%r = extension(%r, parser, %r)" % (msg, key, attrs))
+            if found_flag:
+                continue
+            elif msg:
+                break
+            #else (we found nothing, but also didn't have an error msg) we just continue below:
 
         try:
             eq = parser.get_token()
@@ -804,7 +1050,7 @@ def getUnicodeIndexGroup(name):
     if u'\uAC00' <= c <= u'\uD7AF': # Hangul Syllables
         return unichr(0xac00 + (int(ord(c) - 0xac00) / 588) * 588)
     else:
-        return c
+        return c.upper() # we put lower and upper case words into the same index group
 
 
 def isStrictWikiname(name, word_re=re.compile(ur"^(?:[%(u)s][%(l)s]+){2,}$" % {'u':config.chars_upper, 'l':config.chars_lower})):
@@ -827,23 +1073,31 @@ def isPicture(url):
     @return: true if url points to a picture
     """
     extpos = url.rfind(".")
-    return extpos > 0 and url[extpos:].lower() in ['.gif', '.jpg', '.jpeg', '.png']
+    return extpos > 0 and url[extpos:].lower() in ['.gif', '.jpg', '.jpeg', '.png', '.bmp', '.ico', ]
 
 
 def link_tag(request, params, text=None, formatter=None, on=None, **kw):
     """ Create a link.
 
+    TODO: cleanup css_class
+
     @param request: the request object
     @param params: parameter string appended to the URL after the scriptname/
     @param text: text / inner part of the <a>...</a> link - does NOT get
                  escaped, so you can give HTML here and it will be used verbatim
     @param formatter: the formatter object to use
-    @keyword on: opening/closing tag only
-    @keyword attrs: additional attrs (HTMLified string)
+    @param on: opening/closing tag only
+    @keyword attrs: additional attrs (HTMLified string) (removed in 1.5.3)
     @rtype: string
     @return: formatted link tag
     """
-    css_class = kw.get('css_class', None)
+    if kw.has_key('css_class'):
+        css_class = kw['css_class']
+        del kw['css_class'] # one time is enough
+    else:
+        css_class = None
+    id = kw.get('id', None)
+    name = kw.get('name', None)
     if text is None:
         text = params # default
     if formatter:
@@ -855,12 +1109,14 @@ def link_tag(request, params, text=None, formatter=None, on=None, **kw):
                 formatter.url(0))
     if on != None and not on:
         return '</a>'
-        
+    
     attrs = ''
-    if kw.has_key('attrs'):
-        attrs += ' ' + kw['attrs']
     if css_class:
         attrs += ' class="%s"' % css_class
+    if id:
+        attrs += ' id="%s"' % id
+    if name:
+        attrs += ' name="%s"' % name
     result = '<a%s href="%s/%s">' % (attrs, request.getScriptname(), params)
     if on:
         return result
@@ -979,6 +1235,7 @@ def send_title(request, text, **keywords):
     @keyword pagename: 'PageName'
     @keyword page: the page instance that called us.
     @keyword print_mode: 1 (or 0)
+    @keyword editor_mode: 1 (or 0)
     @keyword media: css media type, defaults to 'screen'
     @keyword allow_doubleclick: 1 (or 0)
     @keyword html_head: additional <head> code
@@ -1007,7 +1264,7 @@ def send_title(request, text, **keywords):
     page_user_prefs = getSysPage(request, 'UserPreferences').page_name
     page_help_formatting = getSysPage(request, 'HelpOnFormatting').page_name
     page_find_page = getSysPage(request, 'FindPage').page_name
-    page_home_page = getattr(getHomePage(request), 'page_name', None)
+    home_page = getInterwikiHomePage(request) # XXX sorry theme API change!!! Either None or tuple (wikiname,pagename) now.
     page_parent_page = getattr(page.getParentPage(), 'page_name', None)
     
     # Prepare the HTML <head> element
@@ -1030,13 +1287,17 @@ def send_title(request, text, **keywords):
         user_head.append(request.cfg.html_head_queries)
     elif request.request_method == 'POST':
         user_head.append(request.cfg.html_head_posts)
+    # we don't want to have BadContent stuff indexed:
+    elif pagename in ['BadContent', 'LocalBadContent', ]:
+        user_head.append(request.cfg.html_head_posts)
     # if it is a special page, index it and follow the links - we do it
     # for the original, English pages as well as for (the possibly
     # modified) frontpage:
     elif pagename in [page_front_page, request.cfg.page_front_page,
                       page_title_index, 'TitleIndex',
                       page_find_page, 'FindPage',
-                      page_site_navigation, 'SiteNavigation',]:
+                      page_site_navigation, 'SiteNavigation',
+                      'RecentChanges',]:
         user_head.append(request.cfg.html_head_index)
     # if it is a normal page, index it, but do not follow the links, because
     # there are a lot of illegal links (like actions) or duplicates:
@@ -1057,13 +1318,14 @@ def send_title(request, text, **keywords):
 %s
 """ % (
         ''.join(user_head),
-        keywords.get('html_head', ''),
         request.theme.html_head({
+            'page': page,
             'title': escape(text),
             'sitename': escape(request.cfg.html_pagetitle or request.cfg.sitename),
             'print_mode': keywords.get('print_mode', False),
             'media': keywords.get('media', 'screen'),
-        })
+        }),
+        keywords.get('html_head', ''),
     ))
 
     # Links
@@ -1108,7 +1370,7 @@ def send_title(request, text, **keywords):
         '<link rel="Glossary" href="%s/%s">\n' % (scriptname, quoteWikinameURL(page_word_index)),
         '<link rel="Help" href="%s/%s">\n' % (scriptname, quoteWikinameURL(page_help_formatting)),
                   ])
-
+    
     output.append("</head>\n")
     request.write(''.join(output))
     output = []
@@ -1125,10 +1387,10 @@ def send_title(request, text, **keywords):
         not keywords.get('print_mode', 0) and
         request.user.edit_on_doubleclick):
         if request.user.may.write(pagename): # separating this gains speed
-            querystr = escape(util.web.makeQueryString({'action': 'edit'}))
-            # TODO: remove escape=0 in 1.4
+            querystr = escape(makeQueryString({'action': 'edit'}))
+            # TODO: remove escape=0 in 2.0
             url = page.url(request, querystr, escape=0)
-            bodyattr.append(''' ondblclick="location.href='%s'"''' % url)
+            bodyattr.append(''' ondblclick="location.href='%s'" ''' % url)
 
     # Set body to the user interface language and direction
     bodyattr.append(' %s' % request.theme.ui_lang_attr())
@@ -1144,9 +1406,15 @@ def send_title(request, text, **keywords):
     
     # If in print mode, start page div and emit the title
     if keywords.get('print_mode', 0):
-        d = {'title_text': text, 'title_link': None, 'page': page,}
+        d = {
+            'title_text': text,
+            'title_link': None,
+            'page': page,
+            'page_name': pagename or '',
+        }
         request.themedict = d
         output.append(theme.startPage())
+        output.append(theme.interwiki(d))      
         output.append(theme.title(d))      
 
     # In standard mode, emit theme.header
@@ -1165,7 +1433,7 @@ def send_title(request, text, **keywords):
             'page_name': pagename or '',
             'page_find_page': page_find_page,
             'page_front_page': page_front_page,
-            'page_home_page': page_home_page,
+            'home_page': home_page,
             'page_help_contents': page_help_contents,
             'page_help_formatting': page_help_formatting,
             'page_parent_page': page_parent_page,
@@ -1196,7 +1464,10 @@ def send_title(request, text, **keywords):
         request.themedict = d
 
         # now call the theming code to do the rendering
-        output.append(theme.header(d))
+        if keywords.get('editor_mode', 0):
+            output.append(theme.editorheader(d))
+        else:
+            output.append(theme.header(d))
     
     # emit it
     request.write(''.join(output))
@@ -1210,8 +1481,6 @@ def send_footer(request, pagename, **keywords):
 
     @param request: the request object
     @param pagename: WikiName of the page
-    @keyword editable: true, when page is editable (default: true)
-    @keyword showpage: true, when link back to page is wanted (default: false)
     @keyword print_mode: true, when page is displayed in Print mode
     """
     d = request.themedict
@@ -1222,8 +1491,6 @@ def send_footer(request, pagename, **keywords):
         request.write(theme.pageinfo(d['page']))
         request.write(theme.endPage())
     else:
-        # This is used only by classic now, kill soon
-        d['footer_fragments'] = request._footer_fragments
         request.write(theme.footer(d, **keywords))
 
     
@@ -1231,25 +1498,37 @@ def send_footer(request, pagename, **keywords):
 ### Tickets - used by RenamePage and DeletePage
 ########################################################################
 
-def createTicket(tm = None):
+def createTicket(request, tm=None):
     """Create a ticket using a site-specific secret (the config)"""
-    import sha, time, types
+    import sha
     ticket = tm or "%010x" % time.time()
     digest = sha.new()
     digest.update(ticket)
 
-    cfgvars = vars(config)
-    for var in cfgvars.values():
-        if type(var) is types.StringType:
+    varnames = ['data_dir', 'data_underlay_dir', 'language_default',
+                'mail_smarthost', 'mail_from', 'page_front_page',
+                'theme_default', 'sitename', 'logo_string',
+                'interwikiname', 'user_homewiki', 'acl_rights_before', ]
+    for varname in varnames:
+        var = getattr(request.cfg, varname, None)
+        if isinstance(var, (str, unicode)):
             digest.update(repr(var))
 
     return "%s.%s" % (ticket, digest.hexdigest())
 
 
-def checkTicket(ticket):
+def checkTicket(request, ticket):
     """Check validity of a previously created ticket"""
-    timestamp = ticket.split('.')[0]
-    ourticket = createTicket(timestamp)
+    try:
+        timestamp_str = ticket.split('.')[0]
+        timestamp = int(timestamp_str, 16)
+    except ValueError:
+        # invalid or empty ticket
+        return False
+    now = time.time()
+    if timestamp < now - 10*3600:
+        # we don't accept tickets older than 10h
+        return False
+    ourticket = createTicket(request, timestamp_str)
     return ticket == ourticket
 
-
